@@ -1,9 +1,14 @@
 import asyncio
 import contextlib
 import logging
+import time
 from typing import Callable, Coroutine, Dict, List, Optional, Tuple
 
-from flux_led.protocol import ProtocolLEDENET8Byte, ProtocolLEDENETOriginal
+from flux_led.protocol import (
+    ProtocolLEDENET8Byte,
+    ProtocolLEDENETAddressableA3,
+    ProtocolLEDENETOriginal,
+)
 
 from .aioprotocol import AIOLEDENETProtocol
 from .base_device import PROTOCOL_PROBES, LEDENETDevice
@@ -13,21 +18,36 @@ from .const import (
     COLOR_MODE_RGB,
     COLOR_MODE_RGBW,
     COLOR_MODE_RGBWW,
+    EFFECT_MUSIC,
     EFFECT_RANDOM,
     STATE_BLUE,
     STATE_COOL_WHITE,
     STATE_GREEN,
     STATE_RED,
     STATE_WARM_WHITE,
+    MultiColorEffects,
 )
 from .utils import color_temp_to_white_levels, rgbw_brightness, rgbww_brightness
 
 _LOGGER = logging.getLogger(__name__)
 
-
+COMMAND_SPACING_DELAY = 1
 MAX_UPDATES_WITHOUT_RESPONSE = 4
 POWER_STATE_TIMEOUT = 1.2  # number of seconds before declaring on/off failed
 
+#
+# PUSH_UPDATE_INTERVAL reduces polling the device for state when its off
+# since we do not care about the state when its off. When it turns on
+# the device will push its new state to us anyways (except for buggy firmwares
+# are identified in protocol.py)
+#
+# The downside to a longer polling interval for OFF is the
+# time to declare the device offline is MAX_UPDATES_WITHOUT_RESPONSE*PUSH_UPDATE_INTERVAL
+#
+PUSH_UPDATE_INTERVAL = 90  # seconds
+
+NEVER_TIME = -PUSH_UPDATE_INTERVAL
+
 
 class AIOWifiLedBulb(LEDENETDevice):
     """A LEDENET Wifi bulb device."""
@@ -37,11 +57,15 @@ class AIOWifiLedBulb(LEDENETDevice):
         super().__init__(ipaddr, port, timeout)
         self._lock = asyncio.Lock()
         self._aio_protocol: Optional[AIOLEDENETProtocol] = None
+        self._ic_future: "asyncio.Future[bool]" = asyncio.Future()
         self._on_futures: List["asyncio.Future[bool]"] = []
         self._off_futures: List["asyncio.Future[bool]"] = []
-        self._data_future: Optional["asyncio.Future[bytes]"] = None
+        self._determine_protocol_future: Optional["asyncio.Future[bool]"] = None
         self._updated_callback: Optional[Callable[[], None]] = None
         self._updates_without_response = 0
+        self._pixels_per_segment: Optional[int] = None
+        self._segments: Optional[int] = None
+        self._last_update_time: float = NEVER_TIME
         self._buffer = b""
         self.loop = asyncio.get_running_loop()
 
@@ -49,6 +73,16 @@ class AIOWifiLedBulb(LEDENETDevice):
         """Setup the connection and fetch initial state."""
         self._updated_callback = updated_callback
         await self._async_determine_protocol()
+        assert self._protocol is not None
+        if not self._protocol.zones:
+            return
+        assert isinstance(self._protocol, ProtocolLEDENETAddressableA3)
+        await self._async_send_msg(self._protocol.construct_request_strip_setting())
+        try:
+            await asyncio.wait_for(self._ic_future, timeout=self.timeout)
+        except asyncio.TimeoutError:
+            self.set_unavailable()
+            raise RuntimeError("Could not determine number pixels")
 
     async def async_stop(self) -> None:
         """Shutdown the connection"""
@@ -94,7 +128,12 @@ class AIOWifiLedBulb(LEDENETDevice):
 
     async def async_turn_on(self) -> bool:
         """Turn on the device."""
-        calls = (self._async_turn_on, self._async_turn_off_on, self._async_turn_on)
+        calls = (
+            self._async_turn_on,
+            self._async_turn_off_on,
+            self._async_turn_on,
+            self._async_turn_on,
+        )
         for idx, call in enumerate(calls):
             if (
                 await self._async_execute_and_wait_for(self._on_futures, call)
@@ -116,7 +155,12 @@ class AIOWifiLedBulb(LEDENETDevice):
 
     async def async_turn_off(self) -> bool:
         """Turn off the device."""
-        calls = (self._async_turn_off, self._async_turn_on_off, self._async_turn_off)
+        calls = (
+            self._async_turn_off,
+            self._async_turn_on_off,
+            self._async_turn_off,
+            self._async_turn_off,
+        )
         for idx, call in enumerate(calls):
             if (
                 await self._async_execute_and_wait_for(self._off_futures, call)
@@ -140,6 +184,21 @@ class AIOWifiLedBulb(LEDENETDevice):
 
         The callback will be triggered when the state is recieved.
         """
+        now = time.monotonic()
+        assert self._protocol is not None
+        if (self._last_update_time + PUSH_UPDATE_INTERVAL) > now:
+            if self.is_on:
+                # If the device pushes state updates when on
+                # then no need to poll except for the interval
+                # to make sure the device is still responding
+                if self._protocol.state_push_updates:
+                    return
+            elif self._protocol.power_push_updates:
+                # If the device pushes power updates
+                # then no need to poll except for the interval
+                # to make sure the device is still responding
+                return
+        self._last_update_time = now
         if self._updates_without_response == MAX_UPDATES_WITHOUT_RESPONSE:
             if self._aio_protocol:
                 self._aio_protocol.close()
@@ -207,10 +266,58 @@ class AIOWifiLedBulb(LEDENETDevice):
         if effect == EFFECT_RANDOM:
             await self.async_set_random()
             return
+        if effect == EFFECT_MUSIC:
+            await self.async_set_music_mode(brightness=brightness)
+            return
         await self.async_set_preset_pattern(
             self._effect_to_pattern(effect), speed, brightness
         )
 
+    async def async_set_zones(
+        self,
+        rgb_list: List[Tuple[int, int, int]],
+        speed: int,
+        effect: MultiColorEffects,
+    ) -> None:
+        """Set zones."""
+        assert self._protocol is not None
+        if not self._protocol.zones:
+            raise ValueError("{self.model} does not support zones")
+        assert self._pixels_per_segment is not None
+        assert isinstance(self._protocol, ProtocolLEDENETAddressableA3)
+        await self._async_send_msg(
+            self._protocol.construct_zone_change(
+                self._pixels_per_segment, rgb_list, speed, effect
+            )
+        )
+
+    async def async_set_music_mode(
+        self,
+        sensitivity: int = 100,
+        brightness: int = 100,
+        mode: Optional[int] = None,
+        effect: Optional[int] = None,
+        foreground_colors: Optional[Tuple[int, int, int]] = None,
+        background_colors: Optional[Tuple[int, int, int]] = None,
+    ) -> None:
+        """Set music mode."""
+        assert self._protocol is not None
+        if not self.microphone:
+            raise ValueError("{self.model} does not have a built-in microphone")
+        for idx, bytes_send in enumerate(
+            self._protocol.construct_music_mode(
+                sensitivity,
+                brightness,
+                mode,
+                effect,
+                foreground_colors or self.rgb,
+                background_colors,
+            )
+        ):
+            if idx > 0:
+                await asyncio.sleep(COMMAND_SPACING_DELAY)
+            await self._async_send_msg(bytes_send)
+
     async def async_set_random(self) -> None:
         """Set levels randomly."""
         await self._async_process_levels_change(*self._generate_random_levels_change())
@@ -281,21 +388,30 @@ class AIOWifiLedBulb(LEDENETDevice):
                 )
             self._async_process_message(msg)
 
+    def _async_process_state_response(self, msg: bytes) -> bool:
+        if (
+            self._determine_protocol_future
+            and not self._determine_protocol_future.done()
+        ):
+            assert self._protocol is not None
+            self._set_protocol_from_msg(msg, self._protocol.name)
+            self._determine_protocol_future.set_result(True)
+        return self.process_state_response(msg)
+
     def _async_process_message(self, msg: bytes) -> None:
         """Process a full message (maybe reassembled)."""
-        if self._data_future and not self._data_future.done():
-            self._data_future.set_result(msg)
-            return
         assert self._protocol is not None
         self.set_available()
         assert self._updated_callback is not None
         prev_state = self.raw_state
         if self._protocol.is_valid_addressable_response(msg):
             self.process_addressable_response(msg)
-        if self._protocol.is_valid_state_response(msg):
-            self.process_state_response(msg)
+        elif self._protocol.is_valid_state_response(msg):
+            self._async_process_state_response(msg)
         elif self._protocol.is_valid_power_state_response(msg):
             self.process_power_state_response(msg)
+        elif self._protocol.is_valid_ic_response(msg):
+            self.process_ic_response(msg)
         else:
             return
         if self.raw_state == prev_state:
@@ -305,7 +421,31 @@ class AIOWifiLedBulb(LEDENETDevice):
             if not future.done():
                 future.set_result(True)
         futures.clear()
-        self._updated_callback()
+        try:
+            self._updated_callback()
+        except Exception as ex:  # pylint: disable=broad-except
+            _LOGGER.error("Error while calling callback: %s", ex)
+
+    def process_ic_response(self, msg: bytes) -> bool:
+        assert self._aio_protocol is not None
+        high_byte = msg[2]
+        low_byte = msg[3]
+        self._pixels_per_segment = (high_byte << 8) + low_byte
+        _LOGGER.debug(
+            "Pixel count (high: %s, low: %s) is: %s",
+            hex(high_byte),
+            hex(low_byte),
+            self._pixels_per_segment,
+        )
+        self._segments = msg[5]
+        _LOGGER.debug(
+            "Segment count (%s) is: %s",
+            hex(msg[5]),
+            self._segments,
+        )
+        if not self._ic_future.done():
+            self._ic_future.set_result(True)
+        return True
 
     def process_addressable_response(self, msg: bytes) -> bool:
         assert self._aio_protocol is not None
@@ -315,7 +455,7 @@ class AIOWifiLedBulb(LEDENETDevice):
             " ".join(f"0x{x:02X}" for x in msg[10:-1]),
             len(msg[10:-1]),
         )
-        return self.process_state_response(msg[10:-1])
+        return self._async_process_state_response(msg[10:-1])
 
     async def _async_send_msg(self, msg: bytearray) -> None:
         """Write a message on the socket."""
@@ -334,21 +474,16 @@ class AIOWifiLedBulb(LEDENETDevice):
             async with self._lock:
                 await self._async_connect()
                 assert self._aio_protocol is not None
-                self._data_future = asyncio.Future()
+                self._determine_protocol_future = asyncio.Future()
                 self._aio_protocol.write(protocol.construct_state_query())
                 try:
-                    full_msg = await asyncio.wait_for(
-                        self._data_future, timeout=self.timeout
+                    await asyncio.wait_for(
+                        self._determine_protocol_future, timeout=self.timeout
                     )
                 except asyncio.TimeoutError:
                     self._aio_protocol.close()
                     continue
-                if not protocol.is_valid_state_response(full_msg):
-                    # We just sent a garage query which the old procotol
-                    # cannot process, recycle the connection
-                    self._aio_protocol.close()
-                    continue
-                self._set_protocol_from_msg(full_msg, protocol.name)
-                self.process_state_response(full_msg)
-                return
+                else:
+                    return
+        self.set_unavailable()
         raise RuntimeError("Cannot determine protocol")
