@@ -15,7 +15,16 @@ class LevelWriteMode(Enum):
     WHITES = 0x0F
 
 
+class MultiColorEffects(Enum):
+    STATIC = 0x01
+    RUNNING_WATER = 0x02
+    STROBE = 0x03
+    JUMP = 0x04
+    BREATHING = 0x05
+
+
 PRESET_MUSIC_MODE: Final = 0x62
+PRESET_MUSIC_MODE_LEGACY: Final = 0x5D
 
 ATTR_IPADDR: Final = "ipaddr"
 ATTR_ID: Final = "id"
@@ -103,3 +112,4 @@ CHANNEL_STATES = {
 
 
 EFFECT_RANDOM = "random"
+EFFECT_MUSIC = "music"
