@@ -1,6 +1,7 @@
 """FluxLED Models Database."""
 
 from dataclasses import dataclass
+from enum import Enum, auto
 from typing import Dict, List, Optional, Set
 
 from .const import (
@@ -19,8 +20,10 @@ from .const import (
 )
 from .protocol import (
     PROTOCOL_LEDENET_8BYTE,
+    PROTOCOL_LEDENET_8BYTE_AUTO_ON,
     PROTOCOL_LEDENET_8BYTE_DIMMABLE_EFFECTS,
     PROTOCOL_LEDENET_9BYTE,
+    PROTOCOL_LEDENET_9BYTE_AUTO_ON,
     PROTOCOL_LEDENET_9BYTE_DIMMABLE_EFFECTS,
     PROTOCOL_LEDENET_ADDRESSABLE_A1,
     PROTOCOL_LEDENET_ADDRESSABLE_A2,
@@ -63,6 +66,20 @@ class LEDENETModel:
         return protocol
 
 
+class LEDENETChip(Enum):
+    ESP8266 = auto()  # aka ESP8285
+    BL602 = auto()  # supports BLE as well
+    S82GESNC = auto()
+    HFLPB100 = auto()
+
+
+@dataclass
+class LEDENETHardware:
+    model: str  # The model string
+    chip: LEDENETChip
+    rf_remote: bool
+
+
 BASE_MODE_MAP = {
     0x01: {COLOR_MODE_DIM},
     0x02: {COLOR_MODE_CCT},
@@ -123,6 +140,108 @@ GENERIC_RGBWW_MAP = {
 
 UNKNOWN_MODEL = "Unknown Model"
 
+# Assumed model version scheme
+#
+# Example AK001-ZJ2149
+#
+#  0  1  2  3  4  5
+#  Z  J  2  1  4  9
+#  |  |  |  |  |  |
+#  |  |  |  |  |  |
+#  |  |  |  |  |  Minor Version
+#  |  |  |  |  Major Version
+#  |  |  |  Chip
+#  |  |  Generation
+#  |  Unknown
+#  Zengge
+#
+HARDWARE = [
+    LEDENETHardware(
+        model="AK001-ZJ100",
+        chip=LEDENETChip.ESP8266,
+        rf_remote=False,  # verified
+    ),
+    LEDENETHardware(
+        model="AK001-ZJ200",
+        chip=LEDENETChip.ESP8266,
+        rf_remote=False,  # verified
+    ),
+    LEDENETHardware(
+        model="AK001-ZJ210",  # might also be "AK001-ZJ2100"
+        chip=LEDENETChip.ESP8266,  # verified
+        rf_remote=False,  # verified
+    ),
+    LEDENETHardware(
+        model="AK001-ZJ2101",
+        chip=LEDENETChip.ESP8266,
+        rf_remote=False,
+    ),
+    LEDENETHardware(
+        model="AK001-ZJ2104",
+        chip=LEDENETChip.ESP8266,
+        rf_remote=False,  # verified
+    ),
+    LEDENETHardware(
+        model="AK001-ZJ2134",  # seen in smart plugs only?
+        chip=LEDENETChip.S82GESNC,  # couldn't get the device appart
+        rf_remote=False,
+    ),
+    LEDENETHardware(
+        model="AK001-ZJ2145",
+        chip=LEDENETChip.BL602,
+        rf_remote=False,  # verified
+    ),
+    LEDENETHardware(
+        model="AK001-ZJ2146",  # seen in smart plugs & Controller RGBCW, but RF remote isn't supported on plugs
+        chip=LEDENETChip.BL602,  # verified
+        rf_remote=True,  # verified
+    ),
+    LEDENETHardware(
+        model="AK001-ZJ2147",  # seen on Controller RGBW
+        chip=LEDENETChip.BL602,
+        rf_remote=True,  # verified
+    ),
+    LEDENETHardware(
+        model="AK001-ZJ2148",  # seen on older Addressable v3
+        chip=LEDENETChip.BL602,
+        rf_remote=True,  # verified
+    ),
+    LEDENETHardware(
+        model="AK001-ZJ2149",  # seen on newer Addressable v3
+        chip=LEDENETChip.BL602,
+        rf_remote=True,  # verified
+    ),
+    LEDENETHardware(
+        model="HF-A11-ZJ002",  # reported older large box controllers (may be original proto)
+        chip=LEDENETChip.HFLPB100,
+        rf_remote=False,  # unverified
+    ),
+    LEDENETHardware(
+        model="HF-LPB100",  # reported on older UFO
+        chip=LEDENETChip.HFLPB100,
+        rf_remote=False,  # unverified
+    ),
+    LEDENETHardware(
+        model="HF-LPB100-0",  # reported on older UFO
+        chip=LEDENETChip.HFLPB100,
+        rf_remote=False,  # unverified
+    ),
+    LEDENETHardware(
+        model="HF-LPB100-1",  # reported on older UFO
+        chip=LEDENETChip.HFLPB100,
+        rf_remote=False,  # unverified
+    ),
+    LEDENETHardware(
+        model="HF-LPB100-ZJ002",  # seen on older UFO
+        chip=LEDENETChip.HFLPB100,
+        rf_remote=False,  # unverified
+    ),
+    LEDENETHardware(
+        model="HF-LPB100-ZJ200",  # seen on RGBW Downlight
+        chip=LEDENETChip.HFLPB100,
+        rf_remote=False,  # unverified
+    ),
+]
 
 MODELS = [
     LEDENETModel(
@@ -138,7 +257,13 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x04,
-        models=["AK001-ZJ200"],
+        models=[
+            "HF-LPB100",
+            "HF-LPB100-0",
+            "HF-LPB100-1",
+            "HF-LPB100-ZJ002",
+            "AK001-ZJ200",
+        ],
         description="UFO Controller RGBW",  # AKA ZJ-WFUF-170F
         always_writes_white_and_colors=True,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
@@ -149,8 +274,10 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x06,
+        # "AK001-ZJ2145" == v2
+        # "AK001-ZJ2147" == v3
         # AK001-ZJ2147 has RF remote support
-        models=["AK001-ZJ2145", "AK001-ZJ2147"],
+        models=["AK001-ZJ100", "AK001-ZJ200", "AK001-ZJ2145", "AK001-ZJ2147"],
         description="Controller RGBW",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE_DIMMABLE_EFFECTS)],
@@ -161,6 +288,7 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x07,
+        # "AK001-ZJ2146" == v2
         # AK001-ZJ2146 has RF remote support
         models=["AK001-ZJ2146"],
         description="Controller RGBCW",
@@ -173,7 +301,7 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x08,
-        models=[],
+        models=["AK001-ZJ2147"],
         description="Controller RGB with MIC",
         always_writes_white_and_colors=True,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE_DIMMABLE_EFFECTS)],
@@ -183,9 +311,9 @@ MODELS = [
         microphone=True,
     ),
     LEDENETModel(
-        model_num=0x09,
+        model_num=0x09,  # same as 0xE1
         models=[],
-        description="Ceiling Light CCT",
+        description="High Voltage Ceiling Light CCT",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
         mode_to_color_mode={},
@@ -210,11 +338,16 @@ MODELS = [
         microphone=False,
     ),
     LEDENETModel(
+        # 'AK001-ZJ2104' likely supports turning on by effect/levels set
+        # 'AK001-ZJ2104' is v7
         model_num=0x0E,
         models=["AK001-ZJ2104"],
         description="Floor Lamp RGBCW",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
-        protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_9BYTE)],
+        protocols=[
+            MinVersionProtocol(7, PROTOCOL_LEDENET_9BYTE_AUTO_ON),
+            MinVersionProtocol(0, PROTOCOL_LEDENET_9BYTE),
+        ],
         mode_to_color_mode={0x01: COLOR_MODES_RGB_CCT},
         color_modes=COLOR_MODES_RGB_CCT,
         channel_map={},
@@ -316,10 +449,16 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x21,
-        models=["AK001-ZJ2101", "AK001-ZJ2104"],
+        # 'AK001-ZJ200' is v1 but with new firmware it will change to v2
+        # 'AK001-ZJ2104' is v2
+        # 'AK001-ZJ2104' likely supports turning on by effect/levels set
+        models=["AK001-ZJ200", "AK001-ZJ2101", "AK001-ZJ2104"],
         description="Bulb Dimmable",
         always_writes_white_and_colors=True,  # Verified required with AK001-ZJ200 bulb
-        protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
+        protocols=[
+            MinVersionProtocol(2, PROTOCOL_LEDENET_8BYTE_AUTO_ON),
+            MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE),
+        ],
         mode_to_color_mode={},
         color_modes={COLOR_MODE_DIM},  # Formerly rgbwcapable
         channel_map={STATE_WARM_WHITE: STATE_RED, STATE_RED: STATE_WARM_WHITE},
@@ -327,7 +466,7 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x25,
-        models=["AK001-ZJ200"],
+        models=["HF-LPB100-ZJ200", "AK001-ZJ200"],
         description="Controller RGB/WW/CW",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_9BYTE)],
@@ -338,11 +477,24 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x33,
-        models=["AK001-ZJ2145", "AK001-ZJ2146", "AK001-ZJ210"],
+        # 'AK001-ZJ2104' == v7 supports turning on by effect/levels set
+        # 'AK001-ZJ2101' == v8
+        # "AK001-ZJ2145" == v9 # no rf support!
+        # "AK001-ZJ2146" == v10 # rf support
+        # "AK001-ZJ2148" == v10.63 # rf support
+        models=[
+            "AK001-ZJ210",
+            "AK001-ZJ2104",
+            "AK001-ZJ2101",
+            "AK001-ZJ2145",
+            "AK001-ZJ2146",
+            "AK001-ZJ2148",
+        ],
         description="Controller RGB",
         always_writes_white_and_colors=True,  # Formerly rgbwprotocol
         protocols=[
             MinVersionProtocol(9, PROTOCOL_LEDENET_8BYTE_DIMMABLE_EFFECTS),
+            MinVersionProtocol(7, PROTOCOL_LEDENET_8BYTE_AUTO_ON),
             MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE),
         ],
         mode_to_color_mode=GENERIC_RGB_MAP,
@@ -352,12 +504,16 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x35,
-        # AK001-ZJ2146 40w flood light, newer smart bulbs (with RF remote control support)
+        # 'AK001-ZJ2101' and 'AK001-ZJ2104' is v7
+        # 'AK001-ZJ2145' is v8
+        # 'AK001-ZJ2146' is v9
+        # 'AK001-ZJ2146' 40w flood light, newer smart bulbs (with RF remote control support)
         models=["AK001-ZJ2101", "AK001-ZJ2104", "AK001-ZJ2145", "AK001-ZJ2146"],
         description="Bulb RGBCW",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[
             MinVersionProtocol(9, PROTOCOL_LEDENET_9BYTE_DIMMABLE_EFFECTS),
+            MinVersionProtocol(7, PROTOCOL_LEDENET_9BYTE_AUTO_ON),
             MinVersionProtocol(0, PROTOCOL_LEDENET_9BYTE),
         ],
         mode_to_color_mode={0x01: COLOR_MODES_RGB_CCT, 0x17: COLOR_MODES_RGB_CCT},
@@ -367,10 +523,15 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x41,
+        # 'AK001-ZJ2104' is v2
+        # 'AK001-ZJ2104' likely supports turning on by effect/levels set
         models=["AK001-ZJ2104"],
         description="Controller Dimmable",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
-        protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
+        protocols=[
+            MinVersionProtocol(2, PROTOCOL_LEDENET_8BYTE_AUTO_ON),
+            MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE),
+        ],
         mode_to_color_mode={},  # Only mode should be 0x41
         color_modes={COLOR_MODE_DIM},  # Formerly rgbwcapable
         channel_map={STATE_WARM_WHITE: STATE_RED, STATE_RED: STATE_WARM_WHITE},
@@ -378,7 +539,7 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x44,
-        models=[],
+        models=["AK001-ZJ200"],
         description="Bulb RGBW",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
@@ -455,7 +616,7 @@ MODELS = [
     LEDENETModel(
         model_num=0x93,
         models=[],
-        description="Switch 1C",
+        description="Switch",  # 1 channel
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
         mode_to_color_mode={},
@@ -466,7 +627,7 @@ MODELS = [
     LEDENETModel(
         model_num=0x94,
         models=[],
-        description="Switch 1c Watt",
+        description="Switch Watt",  # 1 channel
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
         mode_to_color_mode={},
@@ -477,7 +638,7 @@ MODELS = [
     LEDENETModel(
         model_num=0x95,
         models=[],
-        description="Switch 2c",
+        description="Switch 2 Channel",  # 2 channels
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
         mode_to_color_mode={},
@@ -488,7 +649,7 @@ MODELS = [
     LEDENETModel(
         model_num=0x96,
         models=[],
-        description="Switch 4c",
+        description="Switch 4 Channel",  # 4 channels
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
         mode_to_color_mode={},
@@ -499,7 +660,7 @@ MODELS = [
     LEDENETModel(
         model_num=0x97,  # 0x97
         models=["AK001-ZJ2134"],
-        description="Socket 1c",
+        description="Socket",  # 1 channel
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
         mode_to_color_mode={},
@@ -520,6 +681,7 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0xA2,
+        # 'AK001-ZJ2104' likely supports turning on by effect/levels set
         models=["AK001-ZJ2104"],
         description="Addressable v2",
         always_writes_white_and_colors=False,
@@ -542,6 +704,19 @@ MODELS = [
         channel_map={},
         microphone=True,
     ),
+    LEDENETModel(
+        model_num=0xA4,
+        models=["AK001-ZJ2149"],
+        description="Addressable v4",
+        always_writes_white_and_colors=False,
+        protocols=[
+            MinVersionProtocol(0, PROTOCOL_LEDENET_ADDRESSABLE_A3)
+        ],  # Currently no difference from v3 proto
+        mode_to_color_mode={},
+        color_modes=COLOR_MODES_ADDRESSABLE,
+        channel_map={},
+        microphone=True,
+    ),
     LEDENETModel(
         model_num=0xD1,
         models=[],
@@ -556,12 +731,17 @@ MODELS = [
     LEDENETModel(
         model_num=0xE1,
         models=["AK001-ZJ2104"],
-        description="Ceiling Light",
+        description="Ceiling Light CCT",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
         mode_to_color_mode={},
-        color_modes=set(),  # no color modes -- UNVERIFIED
-        channel_map={},
+        color_modes={COLOR_MODE_CCT},  # Formerly rgbwcapable
+        channel_map={
+            STATE_WARM_WHITE: STATE_RED,
+            STATE_RED: STATE_WARM_WHITE,
+            STATE_COOL_WHITE: STATE_GREEN,
+            STATE_GREEN: STATE_COOL_WHITE,
+        },
         microphone=False,
     ),
     LEDENETModel(
