@@ -1,5 +1,4 @@
 import asyncio
-import contextlib
 from datetime import date
 import logging
 import select
@@ -27,10 +26,17 @@ class FluxLEDDiscovery(TypedDict):
     model_num: Optional[int]
     version_num: Optional[int]
     firmware_date: Optional[date]
-    model_info: Optional[str]
+    model_info: Optional[str]  # contains if IR (and maybe BL) if the device supports IR
     model_description: Optional[str]
 
 
+def merge_discoveries(target: FluxLEDDiscovery, source: FluxLEDDiscovery) -> None:
+    """Merge keys from a second discovery that may be missing from the first one."""
+    for k, v in source.items():
+        if target.get(k) is None:
+            target[k] = v  # type: ignore[misc]
+
+
 def _process_discovery_message(data: FluxLEDDiscovery, decoded_data: str) -> None:
     """Process response from b'HF-A11ASSISTHREAD'
 
@@ -50,7 +56,7 @@ def _process_discovery_message(data: FluxLEDDiscovery, decoded_data: str) -> Non
 
 
 def _process_version_message(data: FluxLEDDiscovery, decoded_data: str) -> None:
-    """Process response from b'AT+LVER\r'
+    r"""Process response from b'AT+LVER\r'
 
     b'+ok=07_06_20210106_ZG-BL\r'
     """
@@ -91,7 +97,12 @@ class BulbScanner:
     BROADCAST_ADDRESS = "<broadcast>"
 
     def __init__(self) -> None:
-        self.found_bulbs: List[FluxLEDDiscovery] = []
+        self._discoveries: Dict[str, FluxLEDDiscovery] = {}
+
+    @property
+    def found_bulbs(self) -> List[FluxLEDDiscovery]:
+        """Return only complete bulb discoveries."""
+        return [info for info in self._discoveries.values() if info["id"]]
 
     def getBulbInfoByID(self, id: str) -> FluxLEDDiscovery:
         for b in self.found_bulbs:
@@ -105,10 +116,7 @@ class BulbScanner:
     def _create_socket(self) -> socket.socket:
         sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
-        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        with contextlib.suppress(Exception):
-            sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
-        sock.bind(("", self.DISCOVERY_PORT))
+        sock.bind(("", 0))
         sock.setblocking(False)
         return sock
 
@@ -164,12 +172,6 @@ class BulbScanner:
         elif "," in decoded_data:
             _process_discovery_message(data, decoded_data)
 
-    def _found_bulbs(
-        self, response_list: Dict[str, FluxLEDDiscovery]
-    ) -> List[FluxLEDDiscovery]:
-        """Return only complete bulb discoveries."""
-        return [info for info in response_list.values() if info["id"]]
-
     def send_discovery_messages(
         self,
         sender: Union[socket.socket, asyncio.DatagramTransport],
@@ -192,7 +194,6 @@ class BulbScanner:
         destination = self._destination_from_address(address)
         # set the time at which we will quit the search
         quit_time = time.monotonic() + timeout
-        response_list: Dict[str, FluxLEDDiscovery] = {}
         found_all = False
         # outer loop for query send
         while not found_all:
@@ -220,9 +221,8 @@ class BulbScanner:
                 except socket.timeout:
                     continue
 
-                if self._process_response(data, addr, address, response_list):
+                if self._process_response(data, addr, address, self._discoveries):
                     found_all = True
                     break
 
-        self.found_bulbs = self._found_bulbs(response_list)
         return self.found_bulbs
